==> zipflinger-3.6.3-sources.jar: com/android/zipflinger/LocalFileHeader.java <==
/*
 * Copyright (C) 2019 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.zipflinger;

import com.android.annotations.NonNull;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

class LocalFileHeader {
    private static final int SIGNATURE = 0x04034b50;

    // Minium version = 2.0 (encoded dec = 20)
    public static final short DEFAULT_VERSION_NEEDED = 0x0014;
    public static final int LOCAL_FILE_HEADER_SIZE = 30;

    // Minimum number of bytes needed to create a virtual zip entry (an entry not present in
    // the Central Directory with name length = 0 and an extra field containing padding data).
    public static final long VIRTUAL_HEADER_SIZE = LOCAL_FILE_HEADER_SIZE;

    public static final short COMPRESSION_NONE = 0;
    public static final short COMPRESSION_DEFLATE = 8;

    static final long VIRTUAL_ENTRY_MAX_SIZE = LOCAL_FILE_HEADER_SIZE + Ints.USHRT_MAX;
    static final long OFFSET_TO_NAME = 26;

    private final byte[] nameBytes;
    private final short compressionFlag;
    private final int crc;
    private final long compressedSize;
    private final long uncompressedSize;
    private final int extraPadding;

    LocalFileHeader(
            byte[] nameBytes,
            short compressionFlag,
            int crc,
            long compressedSize,
            long uncompressedSize,
            int extraPadding) {
        this.nameBytes = nameBytes;
        this.compressionFlag = compressionFlag;
        this.crc = crc;
        this.compressedSize = compressedSize;
        this.uncompressedSize = uncompressedSize;
        this.extraPadding = extraPadding;

        if (extraPadding > Ints.USHRT_MAX) {
            String err = String.format("Padding cannot be more than %s bytes", Ints.USHRT_MAX);
            throw new IllegalStateException(err);
        }
    }


    public static void fillVirtualEntry(@NonNull ByteBuffer virtualEntry) {
        int sizeToFill = virtualEntry.capacity();
        if (sizeToFill < VIRTUAL_HEADER_SIZE) {
            String message = String.format("Not enough space for virtual entry (%d)", sizeToFill);
            throw new IllegalStateException(message);
        }
        virtualEntry.order(ByteOrder.LITTLE_ENDIAN);
        virtualEntry.putInt(SIGNATURE);
        virtualEntry.putShort(DEFAULT_VERSION_NEEDED); // Version needed
        virtualEntry.putShort((short) 0); // general purpose flag
        virtualEntry.putShort(COMPRESSION_NONE);
        virtualEntry.putShort(CentralDirectoryRecord.DEFAULT_TIME);
        virtualEntry.putShort(CentralDirectoryRecord.DEFAULT_DATE);
        virtualEntry.putInt(0); // CRC-32
        virtualEntry.putInt(0); // compressed size
        virtualEntry.putInt(0); // uncompressed size
        virtualEntry.putShort((short) 0); // file name length
        // -2 for the extra length ushort we have to write
        virtualEntry.putShort(Ints.intToUshort(virtualEntry.remaining() - 2)); // extra length
        virtualEntry.rewind();
    }

    public void write(@NonNull ZipWriter writer) throws IOException {
        ByteBuffer extraField = ByteBuffer.allocate(extraPadding);
        int bytesNeeded = LOCAL_FILE_HEADER_SIZE + nameBytes.length + extraField.capacity();

        ByteBuffer buffer = ByteBuffer.allocate(bytesNeeded).order(ByteOrder.LITTLE_ENDIAN);
        buffer.putInt(SIGNATURE);
        buffer.putShort(DEFAULT_VERSION_NEEDED);
        buffer.putShort((short) 0); // general purpose flag
        buffer.putShort(compressionFlag);
        buffer.putShort(CentralDirectoryRecord.DEFAULT_TIME);
        buffer.putShort(CentralDirectoryRecord.DEFAULT_DATE);
        buffer.putInt(crc);
        buffer.putInt(Ints.longToUint(compressedSize));
        buffer.putInt(Ints.longToUint(uncompressedSize));
        buffer.putShort(Ints.intToUshort(nameBytes.length));
        buffer.putShort(Ints.intToUshort(extraField.capacity())); // Extra size
        buffer.put(nameBytes);
        buffer.put(extraField);

        buffer.rewind();
        writer.write(buffer);
    }

    public static long sizeFor(@NonNull Source source) {
        // TODO: Zip64 -> Factor in zip64 extra field requirement
        return LOCAL_FILE_HEADER_SIZE + source.getNameBytes().length;
    }
}

==> zipflinger-4.0.1-sources.jar: com/android/zipflinger/LocalFileHeader.java <==
/*
 * Copyright (C) 2019 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.zipflinger;

import com.android.annotations.NonNull;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

class LocalFileHeader {
    private static final int SIGNATURE = 0x04034b50;

    public static final int LOCAL_FILE_HEADER_SIZE = 30;

    // Minimum number of bytes needed to create a virtual zip entry (an entry not present in
    // the Central Directory with name length = 0 and an extra field containing padding data).
    public static final long VIRTUAL_HEADER_SIZE = LOCAL_FILE_HEADER_SIZE;

    public static final short COMPRESSION_NONE = 0;
    public static final short COMPRESSION_DEFLATE = 8;

    static final long VIRTUAL_ENTRY_MAX_SIZE = LOCAL_FILE_HEADER_SIZE + Ints.USHRT_MAX;
    static final long OFFSET_TO_NAME = 26;

    private final byte[] nameBytes;
    private final short compressionFlag;
    private final int crc;
    private final long compressedSize;
    private final long uncompressedSize;
    private final int extraPadding;

    LocalFileHeader(
            byte[] nameBytes,
            short compressionFlag,
            int crc,
            long compressedSize,
            long uncompressedSize,
            int extraPadding) {
        this.nameBytes = nameBytes;
        this.compressionFlag = compressionFlag;
        this.crc = crc;
        this.compressedSize = compressedSize;
        this.uncompressedSize = uncompressedSize;
        this.extraPadding = extraPadding;

        if (extraPadding > Ints.USHRT_MAX) {
            String err = String.format("Padding cannot be more than %s bytes", Ints.USHRT_MAX);
            throw new IllegalStateException(err);
        }
    }


    public static void fillVirtualEntry(@NonNull ByteBuffer virtualEntry) {
        int sizeToFill = virtualEntry.capacity();
        if (sizeToFill < VIRTUAL_HEADER_SIZE) {
            String message = String.format("Not enough space for virtual entry (%d)", sizeToFill);
            throw new IllegalStateException(message);
        }
        virtualEntry.order(ByteOrder.LITTLE_ENDIAN);
        virtualEntry.putInt(SIGNATURE);
        virtualEntry.putShort((short) 0); // Version needed
        virtualEntry.putShort((short) 0); // general purpose flag
        virtualEntry.putShort(COMPRESSION_NONE);
        virtualEntry.putShort(CentralDirectoryRecord.DEFAULT_TIME);
        virtualEntry.putShort(CentralDirectoryRecord.DEFAULT_DATE);
        virtualEntry.putInt(0); // CRC-32
        virtualEntry.putInt(0); // compressed size
        virtualEntry.putInt(0); // uncompressed size
        virtualEntry.putShort((short) 0); // file name length
        // -2 for the extra length ushort we have to write
        virtualEntry.putShort(Ints.intToUshort(virtualEntry.remaining() - 2)); // extra length
        virtualEntry.rewind();
    }

    public void write(@NonNull ZipWriter writer) throws IOException {
        ByteBuffer extraField = ByteBuffer.allocate(extraPadding);
        int bytesNeeded = LOCAL_FILE_HEADER_SIZE + nameBytes.length + extraField.capacity();

        ByteBuffer buffer = ByteBuffer.allocate(bytesNeeded).order(ByteOrder.LITTLE_ENDIAN);
        buffer.putInt(SIGNATURE);
        buffer.putShort((short) 0); // Version needed
        buffer.putShort((short) 0); // general purpose flag
        buffer.putShort(compressionFlag);
        buffer.putShort(CentralDirectoryRecord.DEFAULT_TIME);
        buffer.putShort(CentralDirectoryRecord.DEFAULT_DATE);
        buffer.putInt(crc);
        buffer.putInt(Ints.longToUint(compressedSize));
        buffer.putInt(Ints.longToUint(uncompressedSize));
        buffer.putShort(Ints.intToUshort(nameBytes.length));
        buffer.putShort(Ints.intToUshort(extraField.capacity())); // Extra size
        buffer.put(nameBytes);
        buffer.put(extraField);

        buffer.rewind();
        writer.write(buffer);
    }

    public static long sizeFor(@NonNull Source source) {
        // TODO: Zip64 -> Factor in zip64 extra field requirement
        return LOCAL_FILE_HEADER_SIZE + source.getNameBytes().length;
    }
}
